/*
 * Copyright (C) Since 2006 JasmineSoft Inc. All Rights Reserved.
 */

package jp.jasminesoft.wagby.tests.t002_workflow;

//import static com.codeborne.selenide.CollectionCondition.*;
import static com.codeborne.selenide.Condition.*;
import static com.codeborne.selenide.Selenide.*;

import static jp.jasminesoft.jfc.test.support.selenide.Operations.*;
import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.*;

import com.codeborne.selenide.Configuration;
import com.codeborne.selenide.WebDriverRunner;

import jp.jasminesoft.jfc.test.support.selenide.WebListModel;
import jp.jasminesoft.jfc.test.support.selenide.models.jfcworkstate.JfcworkstateLp;
import jp.jasminesoft.wagby.tests.models.nenkyu.Nenkyu;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runners.MethodSorters;

/**
 * 年休モデルに対してワークフローのテストを行います。
 */
@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class WorkflowTest {

    /** nenkyu モデル */
    private final Nenkyu nenkyu = new Nenkyu();

    /**
     * テストの前処理を行います。
     */
    @BeforeClass
    public static void setUpBeforeClass() {
        Configuration.baseUrl = "http://localhost:8921/wagby";
        Configuration.browser = WebDriverRunner.CHROME; // 利用するブラウザ
        // Selenium(WebDriver) 用ドライバファイル
        System.setProperty("webdriver.chrome.driver",
                "customize/chromedriver.exe");
    }

    /**
     * テストの後処理を行います。
     */
    @AfterClass
    public static void tearDownAfterClass() {
    }

    /** 申請ユーザーでログオンします。 */
    @Test
    public void test01_申請ユーザーでログオン() {
        // ログオン
        logon("applicationUser", "password");
        // メニュー画面が表示されていることを確認
        pageTitle().shouldHave(exactText("メニュー"));
    }

    /** 年休データを登録します。 */
    @Test
    public void test02_登録() {
        pageTitle().shouldHave(exactText("メニュー"));
        selectMenu("サービス", "年休検索");
        pageTitle().shouldHave(exactText("年休 検索"));

        clickNewButton();
        pageTitle().shouldHave(exactText("年休 新規登録"));

        workflow().participants_id.val("年休申請フロー"); // 申請フロー
        nenkyu.applicant.val("applicationUser"); // 申請者
        nenkyu.dayoff.val("2018-03-08");         // 日付
        nenkyu.starthour.val("15");              // 開始時間
        nenkyu.startminute.val("00");            // 開始分
        nenkyu.endhour.val("17");                // 終了時間
        nenkyu.endminute.val("00");              // 終了分
        nenkyu.hours.val("2");                   // 取得時間
        nenkyu.title.val("3/8(木) 年休申請");    // タイトル
        nenkyu.reason.val("イベント参加のため"); // 理由

        save();
        pageTitle().shouldHave(exactText("年休 詳細表示"));
    }

    /** 年休データを確認します。 */
    @Test
    public void test03_確認() {
        pageTitle().shouldHave(exactText("年休 詳細表示"));

        nenkyu.applicant.shouldHave("applicationUser"); // 申請者
        nenkyu.wfstatus.shouldHave(empty);              // フロー状態
        nenkyu.dayoff.shouldHave("2018-03-08");         // 日付
        nenkyu.starthour.shouldHave("15");              // 開始時間
        nenkyu.startminute.shouldHave("00");            // 開始分
        nenkyu.endhour.shouldHave("17");                // 終了時間
        nenkyu.endminute.shouldHave("00");              // 終了分
        nenkyu.hours.shouldHave("2");                   // 取得時間
        nenkyu.title.shouldHave("3/8(木) 年休申請");    // タイトル
        nenkyu.reason.shouldHave("イベント参加のため"); // 理由

        // 申請状況一覧の1行目のデータを確認。
        JfcworkstateLp record01 = workflow().getWorkstate(1);
        record01.username.shouldHave("申請ユーザー");   // 処理者
        record01.event.shouldHave("新規登録");          // 処理内容
        record01.comment.shouldHave(empty);             // コメント
    }

    /** 年休データを申請します。 */
    @Test
    public void test04_申請() {
        pageTitle().shouldHave(exactText("年休 詳細表示"));

        workflow().comment.val("年休申請します。"); // コメント
        workflow().application();                   // 申請ボタンをクリック。

        pageTitle().shouldHave(exactText("年休 詳細表示"));
    }

    /** 申請後の確認を行います。 */
    @Test
    public void test05_申請後の確認() {
        pageTitle().shouldHave(exactText("年休 詳細表示"));

        // 申請状況一覧の2行目のデータを確認。
        JfcworkstateLp record02 = workflow().getWorkstate(2);
        record02.username.shouldHave("申請ユーザー");    // 処理者
        record02.event.shouldHave("申請");               // 処理内容
        record02.comment.shouldHave("年休申請します。"); // コメント
    }

    /** ログオフを行います。 */
    @Test
    public void test06_ログオフ() {
        // ログオフ
        logoff();
        // ログオン画面が表示されていることを確認
        assertThat(title(), is("Wagby アプリケーション ログオン"));
    }

    /** 承認ユーザーでログオンします。 */
    @Test
    public void test07_承認ユーザーでログオン() {
        // ログオン
        logon("admitUser", "password");
        // メニュー画面が表示されていることを確認
        pageTitle().shouldHave(exactText("メニュー"));
    }

    /** 年休データを表示します。 */
    @Test
    public void test08_表示() {
        pageTitle().shouldHave(exactText("メニュー"));

        selectMenu("サービス", "年休検索");
        pageTitle().shouldHave(exactText("年休 検索"));

        clickListPageButton(); // 一覧画面へ
        pageTitle().shouldHave(exactText("年休 一覧"));

        //一覧表示の先頭データの詳細画面へボタンをクリック。
        new WebListModel<>("nenkyu").get(1).clickDetailButton();
        pageTitle().shouldHave(exactText("年休 詳細表示"));

        // フロー状態が"申請"になっていることを確認。
        nenkyu.wfstatus.shouldHave("申請");

        // その他の項目の確認
        nenkyu.applicant.shouldHave("applicationUser"); // 申請者
        nenkyu.dayoff.shouldHave("2018-03-08");         // 日付
        nenkyu.starthour.shouldHave("15");              // 開始時間
        nenkyu.startminute.shouldHave("00");            // 開始分
        nenkyu.endhour.shouldHave("17");                // 終了時間
        nenkyu.endminute.shouldHave("00");              // 終了分
        nenkyu.hours.shouldHave("2");                   // 取得時間
        nenkyu.title.shouldHave("3/8(木) 年休申請");    // タイトル
        nenkyu.reason.shouldHave("イベント参加のため"); // 理由
    }

    /** 年休データを承認します。 */
    @Test
    public void test09_承認() {
        pageTitle().shouldHave(exactText("年休 詳細表示"));

        workflow().comment.val("許可します。"); // コメント
        workflow().admit();                     // 承認ボタンをクリック。

        pageTitle().shouldHave(exactText("年休 詳細表示"));
    }

    /** 承認後の確認を行います。 */
    @Test
    public void test10_承認後の確認() {
        pageTitle().shouldHave(exactText("年休 詳細表示"));

        // 申請状況一覧の3行目のデータを確認。
        JfcworkstateLp record03 = workflow().getWorkstate(3);
        record03.username.shouldHave("承認ユーザー");    // 処理者
        record03.event.shouldHave("承認");               // 処理内容
        record03.comment.shouldHave("許可します。");     // コメント
    }

    /** ログオフを行います。 */
    @Test
    public void test11_ログオフ() {
        // ログオフ
        logoff();
        // ログオン画面が表示されていることを確認
        assertThat(title(), is("Wagby アプリケーション ログオン"));
    }

    /** 申請ユーザーでログオンします。 */
    @Test
    public void test12_決裁ユーザーでログオン() {
        // ログオン
        logon("approvalUser", "password");
        // メニュー画面が表示されていることを確認
        pageTitle().shouldHave(exactText("メニュー"));
    }

    /** 年休データを表示します。 */
    @Test
    public void test13_表示() {
        pageTitle().shouldHave(exactText("メニュー"));

        selectMenu("サービス", "年休検索");
        pageTitle().shouldHave(exactText("年休 検索"));

        clickListPageButton(); // 一覧画面へ
        pageTitle().shouldHave(exactText("年休 一覧"));

        //一覧表示の先頭データの詳細画面へボタンをクリック。
        new WebListModel<>("nenkyu").get(1).clickDetailButton();
        pageTitle().shouldHave(exactText("年休 詳細表示"));

        // フロー状態が"承認"になっていることを確認。
        nenkyu.wfstatus.shouldHave("承認");

        // その他の項目の確認
        nenkyu.applicant.shouldHave("applicationUser"); // 申請者
        nenkyu.dayoff.shouldHave("2018-03-08");         // 日付
        nenkyu.starthour.shouldHave("15");              // 開始時間
        nenkyu.startminute.shouldHave("00");            // 開始分
        nenkyu.endhour.shouldHave("17");                // 終了時間
        nenkyu.endminute.shouldHave("00");              // 終了分
        nenkyu.hours.shouldHave("2");                   // 取得時間
        nenkyu.title.shouldHave("3/8(木) 年休申請");    // タイトル
        nenkyu.reason.shouldHave("イベント参加のため"); // 理由
    }

    /** 年休データを決裁します。 */
    @Test
    public void test14_決裁() {
        pageTitle().shouldHave(exactText("年休 詳細表示"));

        workflow().comment.val("許可します。"); // コメント
        workflow().admit();                     // 承認ボタンをクリック。

        pageTitle().shouldHave(exactText("年休 詳細表示"));
    }

    /** 決裁後の確認を行います。 */
    @Test
    public void test15_決裁後の確認() {
        pageTitle().shouldHave(exactText("年休 詳細表示"));

        // 申請状況一覧の4行目のデータを確認。
        JfcworkstateLp record04 = workflow().getWorkstate(4);
        record04.username.shouldHave("決裁ユーザー");    // 処理者
        record04.event.shouldHave("承認");               // 処理内容
        record04.comment.shouldHave("許可します。");     // コメント

        // 申請状況一覧の5行目のデータを確認。
        JfcworkstateLp record05 = workflow().getWorkstate(5);
        record05.username.shouldHave("決裁ユーザー");    // 処理者
        record05.event.shouldHave("決裁");               // 処理内容
        record05.comment.shouldHave(empty);              // コメント
    }

    /** ログオフを行います。 */
    @Test
    public void test99_ログオフ() {
        // ログオフ
        logoff();
        // ログオン画面が表示されていることを確認
        assertThat(title(), is("Wagby アプリケーション ログオン"));
    }
}
